@@ -22,6 +22,16 @@ from django.template.base import (
     UNKNOWN_SOURCE,
     Template,
     Origin,
+    VARIABLE_TAG_START,
+    VARIABLE_TAG_END,
+    BLOCK_TAG_START,
+    BLOCK_TAG_END,
+    COMMENT_TAG_START,
+    COMMENT_TAG_END,
+    VARIABLE_ATTRIBUTE_SEPARATOR,
+    FILTER_SEPARATOR,
+    FILTER_ARGUMENT_SEPARATOR,
+    tag_re,
 )
 from django.template.context import BaseContext
 from django.template.defaulttags import URLNode, IfNode, TemplateLiteral
@@ -45,10 +55,10 @@ except ImportError:
     pass
 
 
-__version_info__ = "0.1.5"
-__version__ = "0.1.5"
-version = "0.1.5"
-VERSION = "0.1.5"
+__version_info__ = "0.1.6"
+__version__ = "0.1.6"
+version = "0.1.6"
+VERSION = "0.1.6"
 
 
 def get_version():
@@ -134,6 +144,10 @@ VARIABLE_BLACKLIST = {
     # DEBUG=True and the user's IP is in INTERNAL_IPS
     "debug": ("*",),
     "sql_queries": ("*",),
+    "site_title": ("admin_honeypot/login.html",),
+    "site_header": ("admin_honeypot/login.html",),
+    # Django pipeline templates.
+    "media": ("pipeline/css.html",),
 }  # type: Dict[str, Tuple[Text,...]]
 
 IF_VARIABLE_BLACKLIST = {
@@ -150,6 +164,7 @@ IF_VARIABLE_BLACKLIST = {
         "admin_doc/model_detail.html",
         "admin_doc/view_detail.html",
         "admin_doc/bookmarklets.html",
+        "pipeline/css.html",
     ),
     # accessing admindocs detailed model information has: {% if field.help_text %} - ...
     # in file admin_doc/model_detail.html
@@ -159,6 +174,50 @@ IF_VARIABLE_BLACKLIST = {
     # accessing admindocs views detail will have these in file admin_doc/view_detail.html
     "meta.Context": ("admin_doc/view_detail.html",),
     "meta.Templates": ("admin_doc/view_detail.html",),
+    # Rest framework...
+    "name": ("rest_framework/base.html",),
+    "code_style": ("rest_framework/base.html",),
+    "style.hide_label": (
+        "rest_framework/horizontal/checkbox.html",
+        "rest_framework/horizontal/checkbox_multiple.html",
+        "rest_framework/horizontal/dict_field.html",
+        "rest_framework/horizontal/fieldset.html",
+        "rest_framework/horizontal/input.html",
+        "rest_framework/horizontal/list_field.html",
+        "rest_framework/horizontal/list_fieldset.html",
+        "rest_framework/horizontal/radio.html",
+        "rest_framework/horizontal/select.html",
+        "rest_framework/horizontal/select_multiple.html",
+        "rest_framework/horizontal/textarea.html",
+        "rest_framework/vertical/checkbox.html",
+        "rest_framework/vertical/checkbox_multiple.html",
+        "rest_framework/vertical/dict_field.html",
+        "rest_framework/vertical/fieldset.html",
+        "rest_framework/vertical/input.html",
+        "rest_framework/vertical/list_field.html",
+        "rest_framework/vertical/list_fieldset.html",
+        "rest_framework/vertical/radio.html",
+        "rest_framework/vertical/select.html",
+        "rest_framework/vertical/select_multiple.html",
+        "rest_framework/vertical/textarea.html",
+    ),
+    "style.placeholder": (
+        "rest_framework/horizontal/input.html",
+        "rest_framework/horizontal/textarea.html",
+        "rest_framework/inline/input.html",
+        "rest_framework/inline/textarea.html",
+        "rest_framework/vertical/input.html",
+        "rest_framework/vertical/textarea.html",
+    ),
+    "style.autofocus": (
+        "rest_framework/horizontal/input.html",
+        "rest_framework/inline/input.html",
+        "rest_framework/vertical/input.html",
+    ),
+    # Django pipeline templates.
+    "charset": ("pipeline/css.html",),
+    "async": ("pipeline/js.html",),
+    "defer": ("pipeline/js.html",),
 }  # type: Dict[str, Tuple[Text,...]]
 
 
@@ -185,7 +244,7 @@ def variable_blacklist():
 
 
 def create_exception_with_template_debug(context, part, exception_cls):
-    # type: (Context, Text, Type[TemplateSyntaxError]) -> Tuple[Optional[Text], Optional[Dict[Text, Any]]]
+    # type: (Context, Text, Type[TemplateSyntaxError]) -> Tuple[Text, Dict[Text, Any], List[Text]]
     """
     Between Django 2.0 and Django 3.x at least, painting template exceptions
     can do so via the "wrong" template, which highlights nothing.
@@ -231,6 +290,8 @@ def create_exception_with_template_debug(context, part, exception_cls):
 
     assert len(contexts_to_search) <= len(all_potential_contexts)
 
+    template_names = []  # type: List[Text]
+
     for parent in contexts_to_search:
         if isinstance(parent, Origin):
             _template, _origin = context.template.engine.find_template(
@@ -240,39 +301,64 @@ def create_exception_with_template_debug(context, part, exception_cls):
             _template = parent
             _origin = parent.origin
 
-        # Because Django doesn't allow multi-line stuff, we know if we don't
-        # see one of those we're probably in a block comment? And if we see
-        # #} on this line we're probably in a line comment?
+        if _origin.template_name is None:
+            template_names.append(UNKNOWN_SOURCE)
+        else:
+            template_names.append(_origin.template_name)
+
         src = _template.source  # type: Text
-        lines = src.splitlines()  # type: List[Text]
-        for line in lines:
-            if part not in line:
-                continue
-            if "}}" not in line and "%}" not in line:
+
+        # We don't want {{ username }} to be highlighted for an error related to {{ name }}
+        # so we check the previous/next character from the token's match
+        preceeded_by = (
+            VARIABLE_ATTRIBUTE_SEPARATOR,
+            FILTER_SEPARATOR,
+            FILTER_ARGUMENT_SEPARATOR,
+            " ",
+            "{",
+        )
+        proceeded_by = (
+            VARIABLE_ATTRIBUTE_SEPARATOR,
+            FILTER_SEPARATOR,
+            FILTER_ARGUMENT_SEPARATOR,
+            " ",
+            "=",
+            "}",
+        )
+
+        for match in tag_re.finditer(src):
+            match_start, match_end = match.span()
+            token = src[match_start:match_end]
+            if part not in token:
                 continue
 
-            # Line comment wrapping over it.
-            active_line_start = line.find(part)  # type: int
             # We need to make sure that the {# and #} appear to the left & right of our part
-            if (
-                "{#" in line
-                and "#}" in line
-                and line.find("{#") < active_line_start
-                and line.find("#}") > active_line_start
-            ):
+            if VARIABLE_TAG_END not in token and BLOCK_TAG_END not in token:
+                continue
+            elif token[0:2] == COMMENT_TAG_START and token[-2:] == COMMENT_TAG_END:
                 continue
 
-            line_start = src.find(line)  # type: int
-            start = src.find(part, line_start)  # type: int
+            # Where does the line/token start in the original, newlines ridden source?
+            first_occurance_of_token = src.find(token, match_start - 1)  # type: int
+            start = src.find(part, first_occurance_of_token)
+
             if start > -1:
                 end = start + len(part)
+                if src[start - 1] not in preceeded_by:
+                    continue
+                elif src[end] not in proceeded_by:
+                    continue
+
                 exc_info = _template.get_exception_info(
                     exception_cls("ignored"), faketoken(position=(start, end))
                 )  # type: Dict[Text, Any]
-                template_name = _origin.template_name  # type: Optional[Text]
+                if _origin.template_name is None:
+                    template_name = UNKNOWN_SOURCE
+                else:
+                    template_name = _origin.template_name
                 del _template, _origin, start, end
-                return template_name, exc_info
-    return None, None
+                return template_name, exc_info, template_names
+    return UNKNOWN_SOURCE, {}, template_names
 
 
 def new_resolve_lookup(self, context):
@@ -293,11 +379,13 @@ def new_resolve_lookup(self, context):
         whitelisted_by_template = len(whitelisted) > 0
         if not_blacklisted or whitelisted_by_template:
             try:
-                template_name, exc_info = create_exception_with_template_debug(
+                (
+                    template_name,
+                    exc_info,
+                    all_template_names,
+                ) = create_exception_with_template_debug(
                     context, whole_var, MissingVariable
                 )
-                if not template_name:
-                    template_name = UNKNOWN_SOURCE
             except Exception as e2:
                 logger.warning(
                     "failed to create template_debug information", exc_info=e2
@@ -306,7 +394,8 @@ def new_resolve_lookup(self, context):
                 # just carry on and let Django try for itself to set up relevant
                 # debug info
                 template_name = UNKNOWN_SOURCE
-                exc_info = None
+                exc_info = {}
+                all_template_names = [UNKNOWN_SOURCE]
             bit = e.params[0]  # type: Text
             current = e.params[1]
 
@@ -349,18 +438,32 @@ def new_resolve_lookup(self, context):
                 msg += "\nPossibly you meant one of: '{closest_matches}'."
             elif closest:
                 msg += "\nPossibly you meant to use '{closest_matches}'."
-            msg += "\nYou may silence this by adding '{var}' to settings.SHOUTY_VARIABLE_BLACKLIST"
+            msg += "\nYou may silence this globally by adding '{var}' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable."
+            if len(all_template_names) > 1:
+                msg += "\nYou may silence this occurance only by adding one of '{templates}' to the '{var}' key to the settings.SHOUTY_VARIABLE_BLACKLIST iterable."
+            elif all_template_names and UNKNOWN_SOURCE not in all_template_names:
+                msg += "\nYou may silence this occurance only by adding {template} to the '{var}' key to the settings.SHOUTY_VARIABLE_BLACKLIST iterable."
             msg = msg.format(
                 token=bit,
                 var=whole_var,
                 template=template_name,
-                closest_matches="', '".join(closest),
+                closest_matches="', '".join(closest),  # type: ignore
+                templates="', '".join(all_template_names),
             )
             exc = MissingVariable(msg)
-            if context.template.engine.debug and exc_info is not None:
+            if context.template.engine.debug and exc_info:
                 exc_info["message"] = msg
                 exc.template_debug = exc_info
-            if template_name not in whitelisted and "*" not in whitelisted:
+
+            if "*" in whitelisted:
+                logger.debug("Ignoring %s globally", whole_var)
+            elif template_name in whitelisted:
+                logger.debug("Ignoring %s for template %s", whole_var, template_name)
+            elif any(x in whitelisted for x in all_template_names):
+                logger.debug(
+                    "Ignoring %s for templates %r", whole_var, all_template_names
+                )
+            else:
                 raise exc
         else:
             # Let the VariableDoesNotExist bubble back up to whereever it's
@@ -458,11 +561,13 @@ def new_url_render(self, context):
         key = (str(self.view_name.var), outvar)
         if key not in url_blacklist():
             try:
-                template_name, exc_info = create_exception_with_template_debug(
+                (
+                    template_name,
+                    exc_info,
+                    all_template_names,
+                ) = create_exception_with_template_debug(
                     context, outvar, MissingVariable
                 )
-                if not template_name:
-                    template_name = UNKNOWN_SOURCE
             except Exception as e2:
                 logger.warning(
                     "failed to create template_debug information", exc_info=e2
@@ -471,8 +576,8 @@ def new_url_render(self, context):
                 # just carry on and let Django try for itself to set up relevant
                 # debug info
                 template_name = UNKNOWN_SOURCE
-                exc_info = None
-            msg = "{{% url {token!s} ... as {asvar!s} %}} in template '{template} did not resolve.\nYou may silence this by adding {key!r} to settings.SHOUTY_URL_BLACKLIST".format(
+                exc_info = {}
+            msg = "{{% url {token!s} ... as {asvar!s} %}} in template '{template} did not resolve.\nYou may silence this globally by adding {key!r} to settings.SHOUTY_URL_BLACKLIST".format(
                 token=self.view_name, asvar=outvar, key=key, template=template_name,
             )
             exc = MissingVariable(msg)
@@ -606,21 +711,36 @@ default_app_config = "shouty.Shout"
 
 
 if __name__ == "__main__":
-    from django.test import TestCase, override_settings
+    from unittest import skipIf
+    from contextlib import contextmanager
+    from django.test import TestCase, SimpleTestCase, override_settings
     from django.test.runner import DiscoverRunner
     import django
     from django.conf import settings as test_settings
     from django.utils.functional import SimpleLazyObject
 
+    EXTRA_INSTALLED_APPS = ()  # type: Tuple[Text, ...]
+    try:
+        import admin_honeypot
+
+        EXTRA_INSTALLED_APPS += ("admin_honeypot",)
+    except ImportError:
+        pass
+
     def urlpatterns():
         # type: () -> Tuple[Any, ...]
         from django.urls import path, include
         from django.contrib import admin
 
-        return (
+        patterns = ()  # type: Tuple[Any, ...]
+        if "admin_honeypot" in EXTRA_INSTALLED_APPS:
+            patterns += (path("admin_honeypot/", include("admin_honeypot.urls")),)
+
+        patterns += (
             path("admin/doc/", include("django.contrib.admindocs.urls")),
             path("admin/", admin.site.urls),
         )
+        return patterns
 
     test_settings.configure(
         DATABASES={
@@ -634,7 +754,8 @@ if __name__ == "__main__":
             "django.contrib.sessions",
             "django.contrib.messages",
             "shouty",
-        ),
+        )
+        + EXTRA_INSTALLED_APPS,
         MIDDLEWARE=(
             "django.contrib.sessions.middleware.SessionMiddleware",
             "django.contrib.auth.middleware.AuthenticationMiddleware",
@@ -658,10 +779,57 @@ if __name__ == "__main__":
         SHOUTY_URLS=True,
     )
     django.setup()
-    from django.template import Template as TMPL, Context as CTX
+    from django.template import Template, Context as CTX
     from django.forms import IntegerField
 
-    class BasicUsageTestCase(TestCase):  # type: ignore
+    class TMPL(Template):  # type: ignore
+        def __init__(self, *args, **kwargs):
+            # type: (Any, Any) -> None
+            super(TMPL, self).__init__(*args, **kwargs)
+            self.engine.debug = True
+
+    class CustomAssertions(object):
+        @contextmanager
+        def assertRaisesWithTemplateDebug(
+            self, exception_type, exception_repr, debug_data
+        ):
+            # type: (Type[Exception], Text, Dict[str, Any]) -> Iterator[None]
+            try:
+                yield
+            except exception_type as exc:
+                self.assertIn(str(exception_repr), str(exc))  # type: ignore
+                if not hasattr(exc, "template_debug") or exc.template_debug == {}:  # type: ignore
+                    self.fail("Missing template_debug attribute from {}".format(exc))  # type: ignore
+                template_debug = exc.template_debug  # type: ignore
+                if not debug_data:
+                    self.fail(  # type: ignore
+                        "No data provided to check against {}".format(template_debug)
+                    )
+
+                expected = {}
+                found = {}
+                for expected_key, expected_value in debug_data.items():
+                    self.assertIn(expected_key, set(template_debug.keys()))  # type: ignore
+                    found_value = template_debug[expected_key]
+                    if expected_value != found_value:
+                        expected[expected_key] = expected_value
+                        found[expected_key] = found_value
+
+                other_keys = {}
+                for k, v in template_debug.items():
+                    if k not in found and k not in expected:
+                        if isinstance(v, str) and len(v) > 10:
+                            v = "{}...".format(v[0:10])
+                        other_keys[k] = v
+
+                if expected and found:
+                    self.fail(  # type: ignore
+                        "Found template_debug data {found!r} instead of {expected!r}\nOther possible keys include: {others!r}".format(
+                            found=found, expected=expected, others=other_keys,
+                        )
+                    )
+
+    class BasicUsageTestCase(CustomAssertions, TestCase):  # type: ignore
         def setUp(self):
             # type: () -> None
             from shouty import MissingVariable
@@ -676,12 +844,13 @@ if __name__ == "__main__":
                 this does not work: {{ b }}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Variable 'b' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'be'.\n"
-                "You may silence this by adding 'b' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'b' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"line": 3, "start": 76, "end": 77, "during": "b"},
+            ):
                 t.render(CTX({"a": 1, "be": 2}))
 
         def test_silencing(self):
@@ -733,12 +902,14 @@ if __name__ == "__main__":
                 this does not work: {{ a.b.c }}
                 """
             )
-            exc = (
+            exc = ()
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'c' of 'a.b.c' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'cd'.\n"
-                "You may silence this by adding 'a.b.c' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'a.b.c' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"line": 4, "start": 114, "end": 119, "during": "a.b.c"},
+            ):
                 t.render(CTX({"a": {"b": {"cd": 1}}}))
 
         def test_nested_tokens_on_namedtuple(self):
@@ -747,16 +918,17 @@ if __name__ == "__main__":
                 """
                 this works: {{ a }}
                 this works: {{ a.b }}
-                this does not work: {{ a.b.c }}
+                this does not work: ... {{ a.b.c }}
                 """
             )
             nt = namedtuple("nt", "cd ce cf cg")(cd=1, ce=2, cf=3, cg=4)  # type: ignore
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'c' of 'a.b.c' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant one of: 'cg', 'cf', 'ce'.\n"
-                "You may silence this by adding 'a.b.c' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'a.b.c' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"line": 4, "start": 118, "end": 123, "during": "a.b.c"},
+            ):
                 t.render(CTX({"a": {"b": nt}}))
 
         def test_index(self):
@@ -767,12 +939,13 @@ if __name__ == "__main__":
                 this does not work: {{ a.11 }}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token '11' of 'a.11' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use '1'.\n"
-                "You may silence this by adding 'a.11' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'a.11' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"line": 3, "start": 76, "end": 80, "during": "a.11"},
+            ):
                 t.render(CTX({"a": (1, 2)}))
 
         def test_nested_templates(self):
@@ -789,11 +962,12 @@ if __name__ == "__main__":
                 this won't work: {{ c }}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Variable 'c' in template '<unknown source>' does not resolve.\n"
-                "You may silence this by adding 'c' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'c' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 73, "end": 74, "during": "c"},
+            ):
                 t.render(CTX({"a": 1, "subtemplate": st, "b": 2}))
 
         def test_form_possibilities(self):
@@ -807,12 +981,13 @@ if __name__ == "__main__":
             class MyForm(Form):  # type: ignore
                 example = IntegerField()
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'exampl' of 'form.exampl' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'example'.\n"
-                "You may silence this by adding 'form.exampl' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'form.exampl' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 20, "end": 31, "during": "form.exampl"},
+            ):
                 t.render(CTX({"form": MyForm(data={"example": "1"})}))
 
         def test_model_possibilities(self):
@@ -835,13 +1010,13 @@ if __name__ == "__main__":
                 action_flag=1,
                 change_message="",
             )
-
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'object_i' of 'obj.object_i' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant one of: 'object_id', 'objects', 'object_repr'.\n"
-                "You may silence this by adding 'obj.object_i' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'obj.object_i' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 20, "end": 32, "during": "obj.object_i"},
+            ):
                 t.render(CTX({"obj": example}))
 
         def test_model_related_possibilities(self):
@@ -856,13 +1031,13 @@ if __name__ == "__main__":
             from django.contrib.contenttypes.models import ContentType
 
             user = User.objects.create()
-
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'logentry_se' of 'obj.logentry_se.all' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'logentry_set'.\n"
-                "You may silence this by adding 'obj.logentry_se.all' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'obj.logentry_se.all' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 20, "end": 39, "during": "obj.logentry_se.all"},
+            ):
                 t.render(CTX({"obj": user}))
 
         def test_for_loop(self):
@@ -883,12 +1058,13 @@ if __name__ == "__main__":
                 can_add_cakes = (1, 2, 3)
                 can_add_pastries = (1, 2, 3)
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'can_add_pastry' of 'chef.can_add_pastry' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant one of: 'can_add_pastries', 'can_add_cakes'.\n"
-                "You may silence this by adding 'chef.can_add_pastry' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'chef.can_add_pastry' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 149, "end": 168, "during": "chef.can_add_pastry"},
+            ):
                 t.render(CTX({"chef": Chef()}))
 
         def test_multiple_variables_in_if_stmt_and_only_some_resolve(self):
@@ -905,12 +1081,13 @@ if __name__ == "__main__":
                 can_add_cakes = (1, 2, 3)
                 can_add_pastries = (1, 2, 3)
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'can_add_pastry' of 'chef.can_add_pastry' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant one of: 'can_add_pastries', 'can_add_cakes'.\n"
-                "You may silence this by adding 'chef.can_add_pastry' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'chef.can_add_pastry' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 46, "end": 65, "during": "chef.can_add_pastry"},
+            ):
                 t.render(CTX({"chef": Chef()}))
 
         def test_many_if_variables1(self):
@@ -922,11 +1099,12 @@ if __name__ == "__main__":
                 {% endif %}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Variable 'wheee' in template '<unknown source>' does not resolve.\n"
-                "You may silence this by adding 'wheee' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'wheee' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 59, "end": 64, "during": "wheee"},
+            ):
                 t.render(CTX({"whooo": 0}))
 
         def test_many_if_variables2(self):
@@ -938,11 +1116,12 @@ if __name__ == "__main__":
                 {% endif %}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Variable 'whooo' in template '<unknown source>' does not resolve.\n"
-                "You may silence this by adding 'whooo' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'whooo' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 33, "end": 38, "during": "whooo"},
+            ):
                 t.render(CTX({}))
 
         def test_if_elif(self):
@@ -958,11 +1137,12 @@ if __name__ == "__main__":
                 {% endif %}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Variable 'x' in template '<unknown source>' does not resolve.\n"
-                "You may silence this by adding 'x' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'x' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 133, "end": 134, "during": "x"},
+            ):
                 t.render(CTX({}))
 
         def test_exception_debug_info(self):
@@ -976,7 +1156,6 @@ if __name__ == "__main__":
                 {% endif %}
                 """
             )
-            t.engine.debug = True
             try:
                 t.render(CTX({"x": 1, "y": 1, "def": 2}))
             except self.MissingVariable as exc:
@@ -984,7 +1163,33 @@ if __name__ == "__main__":
                 self.assertEqual(exc.template_debug["name"], UNKNOWN_SOURCE)
                 self.assertEqual(exc.template_debug["during"], "abc")
 
-    class UrlTestCase(TestCase):  # type: ignore
+        def test_complex_exception_debug_info(self):
+            # type: () -> None
+            t = TMPL(
+                """
+                {# <title>{{ wheee }}</title> #}
+                but this won't: {% include subtemplate with x=1 only %}
+                {{ wheee }} 
+                """
+            )
+            t.origin.template_name = "parent"
+            st = TMPL(
+                """
+                {% if 0 %} {{ userwheee }}
+                {% endif %}
+                """
+            )
+            st.origin.template_name = "child"
+
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
+                "Variable 'wheee' in template 'parent' does not resolve.\n"
+                "You may silence this globally by adding 'wheee' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 141, "end": 146, "during": "wheee"},
+            ):
+                t.render(CTX({"subtemplate": st}))
+
+    class UrlTestCase(CustomAssertions, SimpleTestCase):  # type: ignore
         def setUp(self):
             # type: () -> None
             from shouty import MissingVariable
@@ -998,14 +1203,15 @@ if __name__ == "__main__":
                 {% url "waffle" as wheee %}
                 """
             )
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 '{% url "waffle" ... as wheee %} in template \'<unknown source> did not resolve.\n'
-                "You may silence this by adding ('waffle', 'wheee') to settings.SHOUTY_URL_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding ('waffle', 'wheee') to settings.SHOUTY_URL_BLACKLIST",
+                {"start": 36, "end": 41, "during": "wheee"},
+            ):
                 t.render(CTX())
 
-    class ReadmeExampleTestCase(TestCase):  # type: ignore
+    class ReadmeExampleTestCase(CustomAssertions, SimpleTestCase):  # type: ignore
         def setUp(self):
             # type: () -> None
             from shouty import MissingVariable
@@ -1026,12 +1232,13 @@ if __name__ == "__main__":
                 is_cake_chef = True
                 can_add_cakes = True
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'chef' of 'chef.can_add_cakes' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'sous_chef'.\n"
-                "You may silence this by adding 'chef.can_add_cakes' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'chef.can_add_cakes' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 23, "during": "chef.can_add_cakes", "end": 41},
+            ):
                 t.render(CTX({"sous_chef": Chef()}))
 
         def test_is_cake_chef_renamed_to_is_pastry_king(self):
@@ -1048,11 +1255,12 @@ if __name__ == "__main__":
                 is_pastry_king = 1
                 can_add_cakes = 1
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'is_cake_chef' of 'chef.is_cake_chef' in template '<unknown source>' does not resolve.\n"
-                "You may silence this by adding 'chef.is_cake_chef' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'chef.is_cake_chef' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 94, "during": "chef.is_cake_chef", "end": 111},
+            ):
                 t.render(CTX({"chef": Chef()}))
 
         def test_can_add_cakes_renamed_to_can_add_pastries(self):
@@ -1072,12 +1280,13 @@ if __name__ == "__main__":
                     # type: () -> bool
                     return True
 
-            exc = (
+            with self.assertRaisesWithTemplateDebug(
+                self.MissingVariable,
                 "Token 'can_add_cakes' of 'chef.can_add_cakes' in template '<unknown source>' does not resolve.\n"
                 "Possibly you meant to use 'can_add_pastries'.\n"
-                "You may silence this by adding 'chef.can_add_cakes' to settings.SHOUTY_VARIABLE_BLACKLIST"
-            )
-            with self.assertRaisesMessage(self.MissingVariable, exc):
+                "You may silence this globally by adding 'chef.can_add_cakes' to the settings.SHOUTY_VARIABLE_BLACKLIST iterable.",
+                {"start": 23, "during": "chef.can_add_cakes", "end": 41},
+            ):
                 t.render(CTX({"chef": Chef()}))
 
     class CommonAppsTestCase(TestCase):  # type: ignore
@@ -1144,6 +1353,16 @@ if __name__ == "__main__":
                     response = self.client.get(url, follow=False)
                     self.assertStatusCode(response, 200)
 
+        @skipIf(
+            "admin_honeypot" not in EXTRA_INSTALLED_APPS,
+            "django-admin-honeypot is not installed",
+        )
+        def test_admin_honeypot_should_render_ok(self):
+            # type: () -> None
+            """ Versions <= 1.1.0 don't set 'site_title' or 'site_header' variables """
+            response = self.client.get("/admin_honeypot/login/", follow=False)
+            self.assertStatusCode(response, 200)
+
         def test_example_404(self):
             # type: () -> None
             """ The technical 404 page should not itself cause a 500 error """
@@ -1154,7 +1373,7 @@ if __name__ == "__main__":
                 response = self.client.get("/favicon.ico", follow=False)
                 self.assertStatusCode(response, 404)
 
-    class InternalVariableBlacklistTestCase(TestCase):  # type: ignore
+    class InternalVariableBlacklistTestCase(SimpleTestCase):  # type: ignore
         def test_im_not_an_idiot(self):
             # type: () -> None
             for k, v in VARIABLE_BLACKLIST.items():
@@ -1184,7 +1403,7 @@ if __name__ == "__main__":
                         )
                     )
 
-    class MyPyTestCase(TestCase):  # type: ignore
+    class MyPyTestCase(SimpleTestCase):  # type: ignore
         def test_for_types(self):
             # type: () -> None
             try:
